#version 450

layout (triangles, fractional_odd_spacing, cw) in;

layout (set = 1, binding = 1) uniform UBO 
{
	mat4 projection;
	mat4 model;
	mat4 modelMatrixForNormals;
	float tessAlpha;
} ubo;

layout (set = 2, binding = 0) uniform ShadowmappingUBO
{
	mat4 depthViewProjMatrix;
} shadowmapping_ubo;

layout (location = 0) in vec3 inNormal[];
layout (location = 1) in vec2 inUV[];

layout (location = 0) out VertexData {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 textureCoordinates;
	// Shadowmapping
	vec4 shadowCoordinates;
} tese_out;

// bias matrix for Shadowmapping
// transforms from [-1,1] to [0,1]
const mat4 biasMat = mat4(
	0.5, 0.0, 0.0, 0.0,
	0.0, 0.5, 0.0, 0.0,
	0.0, 0.0, 1.0, 0.0,
	0.5, 0.5, 0.0, 1.0 );


void main(void)
{
    vec4 localPosition = (gl_TessCoord.x * gl_in[0].gl_Position) +
                  (gl_TessCoord.y * gl_in[1].gl_Position) +
                  (gl_TessCoord.z * gl_in[2].gl_Position);
	
	vec4 position_world = ubo.model * localPosition;
	gl_Position = ubo.projection * position_world;
	
	vec3 oNormal = gl_TessCoord.x*inNormal[0] + gl_TessCoord.y*inNormal[1] + gl_TessCoord.z*inNormal[2];
	vec2 oTexCoord = gl_TessCoord.x*inUV[0] + gl_TessCoord.y*inUV[1] + gl_TessCoord.z*inUV[2];

	tese_out.positionWorld = position_world.xyz;
	tese_out.normalWorld =  mat3(ubo.modelMatrixForNormals) * oNormal;
	tese_out.textureCoordinates = oTexCoord;

	tese_out.shadowCoordinates = ( biasMat * shadowmapping_ubo.depthViewProjMatrix * ubo.model ) * vec4(localPosition.xyz, 1.0);
}